!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !MODULE: charpak_mod.F
!
! !DESCRIPTION: Module CHARPAK\_MOD contains routines from the CHARPAK 
!  string and character manipulation package used by GEOS-Chem.
!\\   
!\\
! !INTERFACE: 
!
      MODULE CHARPAK_MOD
! 
! !USES:
!
      IMPLICIT NONE
      PRIVATE
!
! !PUBLIC MEMBER FUNCTIONS:
!
      PUBLIC  :: CNTMAT     
      PUBLIC  :: COPYTXT    
      PUBLIC  :: CSTRIP     
      PUBLIC  :: ISDIGIT    
      PUBLIC  :: STRREPL    
      PUBLIC  :: STRSPLIT   
      PUBLIC  :: STRSQUEEZE 
      PUBLIC  :: TRANLC     
      PUBLIC  :: TRANUC     
      PUBLIC  :: TXT2INUM   
      PUBLIC  :: TXTEXT 
!
! !REMARKS:
!  CHARPAK routines by Robert D. Stewart, 1992.  Subsequent modifications 
!  made for GEOS-CHEM by Bob Yantosca (1998, 2002, 2004).
!
! !REVISION HISTORY:
!  (1 ) Moved "cntmat.f", "copytxt.f", "cstrip.f", "fillstr.f", "txt2inum.f",
!        "txtext.f", into this F90 module for easier bookkeeping 
!        (bmy, 10/15/01)
!  (2 ) Moved "tranuc.f" into this F90 module (bmy, 11/15/01)
!  (3 ) Now divide module header into MODULE PRIVATE, MODULE VARIABLES, and
!        MODULE ROUTINES sections.  Updated comments (bmy, 5/28/02)
!  (4 ) Wrote a new file "strrepl.f", which replaces a character pattern
!        within a string with replacement text.  Moved "tranlc.f" into
!        this module.  Replaced calls to function LENTRIM with F90 
!        intrinsic function LEN_TRIM.  Removed function FILLSTR and
!        replaced it w/ F90 intrinsic REPEAT. (bmy, 6/25/02)
!  (5 ) Added routine STRSPLIT as a wrapper for TXTEXT.  Also added
!        routines STRREPL and STRSQUEEZE. (bmy, 7/30/02)
!  (6 ) Added function ISDIGIT.  Also replace LEN_TRIM with LEN in routine
!        STRREPL, to allow us to replace tabs w/ spaces. (bmy, 7/20/04)
!  20 Nov 2009 - R. Yantosca - Added ProTeX header
!  20 Aug 2013 - R. Yantosca - Removed "define.h", this is now obsolete
!  07 Aug 2017 - R. Yantosca - Add ProTeX headers for subroutines
!EOP
!------------------------------------------------------------------------------
!BOC
      CONTAINS
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: CntMat
!
! !DESCRIPTION: Counts the number of characters in str1 that match
!  a character in str2.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE CntMat(str1,str2,imat)
!
! !INPUT PARAMETERS: 
!
      CHARACTER(LEN=*) str1,str2
!
! !OUTPUT PARAMETERS: 
!
      INTEGER          imat
!
! !REVISION HISTORY:
!     DATE:   JAN. 6, 1995
!     AUTHOR: R.D. STEWART
!     COMMENTS: Revised slightly (2-5-1996) so that trailing
!               blanks in str1 are ignored.  Revised again
!               on 3-6-1996.
!   7 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER L1,L2,i,j
      LOGICAL again

      L1 = MAX(1,LEN_TRIM(str1))
      L2 = LEN(str2)
      imat = 0
      DO i=1,L1
        again = .true.
        j = 1
        DO WHILE (again)
          IF (str2(j:j).EQ.str1(i:i)) THEN
            imat = imat+1
            again = .false.
          ELSEIF (j.LT.L2) THEN
            j=j+1
          ELSE
            again = .false.
          ENDIF
        ENDDO
      ENDDO

      END SUBROUTINE CntMat
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: CopyTxt
!
! !DESCRIPTION: Write all of the characters in str1 into variable
!               str2 beginning at column, col.  If the length of str1
!               + col is longer than the number of characters str2
!               can store, some characters will not be transfered to
!               str2.  Any characters already existing in str2 will
!               will be overwritten.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE CopyTxt(col,str1,str2)
!
! !INPUT PARAMETERS: 
!
      INTEGER          col
      CHARACTER(LEN=*) str1
!
! !OUTPUT PARAMETERS: 
!
      CHARACTER(LEN=*) str2
!
! !REVISION HISTORY:
!     DATE:   DEC. 24, 1993
!     AUTHOR: R.D. STEWART
!   7 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER ilt1,i1,i,j,ic

      i1 = LEN(str2)
      IF (i1.GT.0) THEN
        ilt1 = LEN(str1)
        IF (ilt1.GT.0) THEN
          ic = MAX0(col,1)
          i = 1
          j = ic
          DO WHILE ((i.LE.ilt1).and.(j.LE.i1))
            str2(j:j) = str1(i:i)
            i = i + 1
            j = ic + (i-1)
          ENDDO
        ENDIF
      ENDIF

      END SUBROUTINE CopyTxt
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: Cstrip
!
! !DESCRIPTION: Strip blanks and null characters for the variable TEXT.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE CSTRIP(text)
!
! !INPUT/OUTPUT PARAMETERS: 
!
      CHARACTER(LEN=*) TEXT
!
! !REMARKS:
!  The original "text" is destroyed upon exit.
!
! !REVISION HISTORY:
!      AUTHOR: Robert D. Stewart
!        DATE: May 19, 1992
!  07 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER ilen,iasc,icnt,i
      CHARACTER(LEN=1) ch

      ilen = LEN(text)
      IF (ilen.GT.1) THEN
        icnt = 1
        DO i=1,ilen
          iasc = ICHAR(text(i:i))
          IF ((iasc.GT.32).AND.(iasc.LT.255)) THEN
!           Keep character
            ch = text(i:i)
            text(icnt:icnt) = ch
            icnt = icnt + 1
          ENDIF
        ENDDO
!       Fill remainder of text with blanks
        DO i=icnt,ilen
          text(i:i) = ' '
        ENDDO
      ENDIF

      END SUBROUTINE CSTRIP
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: IsDigit
!
! !DESCRIPTION: Returned as true if ch is a numeric character (i.e., one of
!  the numbers from 0 to 9).
!\\
!\\
! !INTERFACE:
!
      FUNCTION ISDIGIT( ch ) RESULT( LNUM )
!
! !INPUT PARAMETERS: 
!
      CHARACTER(LEN=1) :: ch
!
! !REMARKS:
!  NOTE: Changed name from ISNUM to ISDIGIT (bmy, 7/15/04)
!
! !REVISION HISTORY:
!     DATE:   NOV. 11, 1993
!     AUTHOR: R.D. STEWART
!   7 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!  
      INTEGER iasc
      LOGICAL lnum

      iasc = ICHAR(ch)
      lnum = .FALSE.
      IF ((iasc.GE.48).AND.(iasc.LE.57)) THEN
        lnum = .TRUE.
      ENDIF

      END FUNCTION ISDIGIT
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: StrRepl
!
! !DESCRIPTION: Subroutine StrRepl replaces all instances of PATTERN within
!  a string STR with replacement text REPLTXT. 
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE StrRepl( STR, PATTERN, REPLTXT )
!
! !INPUT PARAMETERS: 
!
      CHARACTER(LEN=*), INTENT(IN)    :: PATTERN   ! Pattern to search for
      CHARACTER(LEN=*), INTENT(IN)    :: REPLTXT   ! Text to replace
!
! !INPUT/OUTPUT PARAMETERS: 
!
      CHARACTER(LEN=*), INTENT(INOUT) :: STR       ! String to be manipulated
!
! !REMARKS:
!  PATTERN and REPLTXT can now have a different number of characters.
!
! !REVISION HISTORY:
!  25 Jun 2002 - R. Yantosca - Initial version
!  07 Aug 2017 - R. Yantosca - Updated algorithm to allow PATTERN and
!                              REPLTXT to have different numbers of characters
!   7 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      ! Local variables
      INTEGER :: I1, I2

      !=================================================================
      ! StrRepl begins here!
      !=================================================================
!-----------------------------------------------------------------------------
! Prior to 8/7/17:
! Remove restriction that PATTERN and REPLXT have to have same # of chars
! (bmy, 8/7/17)
!      ! Error check: make sure PATTERN and REPLTXT have the same # of chars
!      IF ( LEN( PATTERN ) /= LEN( REPLTXT ) ) THEN 
!         WRITE( 6, '(a)' ) REPEAT( '=', 79 )
!         WRITE( 6, '(a)' ) 
!     &    'STRREPL: PATTERN and REPLTXT must have same # of characters!'
!         WRITE( 6, '(a)' ) 'STOP in STRREPL (charpak_mod.f)'
!         WRITE( 6, '(a)' ) REPEAT( '=', 79 )
!         STOP
!      ENDIF
!
!      ! Loop over all instances of PATTERN in STR
!      DO 
!
!         ! I1 is the starting location of PATTERN w/in STR  
!         I1 = INDEX( STR, PATTERN )
!
!         ! If pattern is not found, then return to calling program
!         IF ( I1 < 1 ) RETURN
!
!         ! I2 is the ending location of PATTERN w/in STR
!         I2 = I1 + LEN_TRIM( PATTERN ) - 1
!      
!         ! Replace text
!         STR(I1:I2) = REPLTXT
!          print*, '### str: ', trim(str)
!
!      ENDDO
!-----------------------------------------------------------------------------

      DO 

         ! I1 is the first character that matches the search pattern;
         ! it must be 1 or larger.  Otherwise exit the routine.
         I1 = INDEX( Str, Pattern )        
         IF ( I1 < 1 ) RETURN

         ! Replace the text.  I2 is the starting position of the
         ! string following the point of text replacement.
         I2 = I1 + LEN( Pattern )
         Str = Str(1:I1-1) // ReplTxt // Str(I2:)

      ENDDO

      END SUBROUTINE StrRepl
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: StrSplit
!
! !DESCRIPTION: Subroutine STRSPLIT returns substrings in a string, separated 
!  by a separator character (similar to IDL's StrSplit function).  This is 
!  mainly a convenience wrapper for CHARPAK routine TxtExt.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE StrSplit( STR, SEP, RESULT, N_SUBSTRS )
!
! !INPUT PARAMETERS: 
!
      CHARACTER(LEN=*), INTENT(IN)  :: STR           ! String to be searched
      CHARACTER(LEN=1), INTENT(IN)  :: SEP           ! Separator character
!
! !OUTPUT PARAMETERS: 
!
      CHARACTER(LEN=*), INTENT(OUT) :: RESULT(255)   ! Returned substrings
      INTEGER,          OPTIONAL    :: N_SUBSTRS     ! # of substrings
!
! !REMARKS:
!
! !REVISION HISTORY:
!  11 Jul 2002 - R. Yantosca - Initial version
!  30 Sep 2014 - R. Yantosca - WORD now has 2047 chars for extra-long strings
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER             :: I, IFLAG, COL
      CHARACTER(LEN=2047) :: WORD

      !=================================================================
      ! STRSPLIT begins here!
      !=================================================================

      ! Initialize
      I         = 0
      COL       = 1 
      IFLAG     = 0
      RESULT(:) = ''
      
      ! Loop until all matches found, or end of string
      DO WHILE ( IFLAG == 0 )

         ! Look for strings beteeen separator string
         CALL TXTEXT ( SEP, TRIM( STR ), COL, WORD, IFLAG )

         ! Store substrings in RESULT array
         I         = I + 1
         RESULT(I) = TRIM( WORD )

      ENDDO

      ! Optional argument: return # of substrings found
      IF ( PRESENT( N_SUBSTRS ) ) N_SUBSTRS = I

      END SUBROUTINE StrSplit
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: StrSqueeze
!
! !DESCRIPTION: Subroutine STRSQUEEZE strips white space from both ends of a 
!  string.  White space in the middle of the string (i.e. between characters) 
!  will be preserved as-is.  Somewhat similar (though not exactly) to IDL's 
!  STRCOMPRESS function.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE StrSqueeze( STR )
!
! !INPUT/OUTPUT PARAMETERS: 
!
      CHARACTER(LEN=*), INTENT(INOUT) :: STR   ! String to be squeezed
!
! !REVISION HISTORY:
!  11 Jul 2002 - R. Yantosca - Initial version
!  07 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
      !=================================================================
      ! STRSQUEEZE begins here!
      !=================================================================
      STR = ADJUSTR( TRIM( STR ) )
      STR = ADJUSTL( TRIM( STR ) )

      END SUBROUTINE StrSqueeze
!EOC      
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: TranLc
!
! !DESCRIPTION: Tranlate a character variable to all lowercase letters.
!               Non-alphabetic characters are not affected.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE TRANLC(text)
!
! !INPUT/OUTPUT PARAMETERS: 
!
      CHARACTER(LEN=*) :: text
!
! !REMARKS:
!  The original "text" is destroyed.
!
! !REVISION HISTORY:
!      AUTHOR: Robert D. Stewart
!        DATE: May 19, 1992
!   7 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER iasc,i,ilen

      ilen = LEN(text)
      DO I=1,ilen
        iasc = ICHAR(text(i:i))
        IF ((iasc.GT.64).AND.(iasc.LT.91)) THEN
          text(i:i) = CHAR(iasc+32)
        ENDIF
      ENDDO

      END SUBROUTINE TRANLC
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: TranUc 
!
! !DESCRIPTION: Tranlate a character variable to all upper case letters.
!               Non-alphabetic characters are not affected.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE TRANUC(text)
!
! !INPUT/OUTPUT PARAMETERS: 
!
      CHARACTER(LEN=*) :: text
!
! !REMARKS:
!  The original "text" is destroyed.
!
! !REVISION HISTORY:
!      AUTHOR: Robert D. Stewart
!        DATE: May 19, 1992
!   7 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER iasc,i,ilen

      ilen = LEN(text)
      DO i=1,ilen
        iasc = ICHAR(text(i:i))
        IF ((iasc.GT.96).AND.(iasc.LT.123)) THEN
          text(i:i) = CHAR(iasc-32)
        ENDIF
      ENDDO

      END SUBROUTINE TRANUC
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: Txt2Inum
!
! !DESCRIPTION: Subroutine TXT2INUM attempts to convert the string of 
!  characters in TXT into a integer number.  FMAT is the VALID format 
!  specifier to use in the internal read statement.  IFLG is returned as a 
!  status flag indicating the success or failure of the operation.  
!  IFLG <=0 if the operation was successful, and > 0 if it failed.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE Txt2Inum(fmat,txt,Inum,iflg)
!
! !INPUT PARAMETERS: 
!
      CHARACTER(LEN=*) txt,fmat
!
! !OUTPUT PARAMETERS: 
!
      INTEGER inum
      INTEGER iflg
!
! !REMARKS:
!  Generally, the Fxx.0 format should be used to convert
!  string of characters to a number.
!
! !REVISION HISTORY:
!      AUTHOR: Robert D. Stewart
!        DATE: DEC 24, 1992
!  07 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:

      READ(txt,fmt=fmat,iostat=iflg) inum

      END SUBROUTINE Txt2Inum
!EOC
!------------------------------------------------------------------------------
!                  GEOS-Chem Global Chemical Transport Model                  !
!------------------------------------------------------------------------------
!BOP
!
! !IROUTINE: TxtExt
!
! !DESCRIPTION: TxtExt extracts a sequence of characters from
!               text and transfers them to word.  The extraction
!               procedure uses a set of character "delimiters"
!               to denote the desired sequence of characters.
!               For example if ch=' ', the first character sequence
!               bracketed by blank spaces will be returned in word.
!               The extraction procedure begins in column, col,
!               of TEXT.  If text(col:col) = ch (any character in
!               the character string), the text is returned beginning
!               with col+1 in text (i.e., the first match with ch
!               is ignored).
!\\
!\\
!               After completing the extraction, col is incremented to
!               the location of the first character following the
!               end of the extracted text.
!\\
!\\
!               A status flag is also returned with the following
!               meaning(s)
!\\
!\\
!               IF iflg = -1, found a text block, but no more characters
!                             are available in TEXT
!                  iflg = 0,  task completed sucessfully (normal term)
!                  iflg = 1,  ran out of text before finding a block of
!                             text.
!\\
!\\
! !INTERFACE:
!
      SUBROUTINE TxtExt(ch,text,col,word,iflg)
!
! !INPUT PARAMETERS: 
!
      CHARACTER(LEN=*) :: ch,text
!
! !INPUT/OUTPUT PARAMETERS: 
!
      INTEGER          :: col
!
! !OUTPUT PARAMETERS: 
!
      CHARACTER(LEN=*) :: word
      INTEGER          :: iflg

!
! !REMARKS:
!  TxtExt is short for Text Extraction.  This routine provides a set of 
!  powerful line-by-line text search and extraction capabilities in
!  standard FORTRAN.
!
! !REVISION HISTORY:
!      AUTHOR: Robert D. Stewart
!        DATE: Jan. 1st, 1995
!
!      REVISIONS: FEB 22, 1996.  Slight bug fix (introduced by a
!        (recent = FLIB 1.04) change in the CntMat routine)
!        so that TxtExt correctlyhandles groups of characters
!        delimited by blanks).
!
!      MODIFICATIONS by Bob Yantosca (6/25/02)
!        (1) Replace call to FILLSTR with F90 intrinsic REPEAT
!
!   7 Aug 2017 - R. Yantosca - Added ProTeX header
!EOP
!------------------------------------------------------------------------------
!BOC
!
! !LOCAL VARIABLES:
!
      INTEGER Tmax,T1,T2,imat
      LOGICAL again,prev

!     Length of text
      Tmax = LEN(text)

!     Fill Word with blanks
      WORD = REPEAT( ' ', LEN( WORD ) )
      
      IF (col.GT.Tmax) THEN
!       Text does not contain any characters past Tmax.
!       Reset col to one and return flag = {error condition}
        iflg = 1
        col = 1
      ELSEIF (col.EQ.Tmax) THEN
!       End of TEXT reached
        CALL CntMat(ch,text(Tmax:Tmax),imat)
        IF (imat.EQ.0) THEN
!         Copy character into Word and set col=1
          CALL CopyTxt(1,Text(Tmax:Tmax),Word)
          col = 1
          iflg = -1
        ELSE
!         Same error condition as if col.GT.Tmax
          iflg = 1
        ENDIF
      ELSE
!       Make sure column is not less than 1
        IF (col.LT.1) col=1
        CALL CntMat(ch,text(col:col),imat)
        IF (imat.GT.0) THEN
          prev=.true.
        ELSE
          prev=.false.
        ENDIF
        T1=col
        T2 = T1

        again = .true.
        DO WHILE (again)
!         Check for a match with a character in ch
          CALL CntMat(ch,text(T2:T2),imat)
          IF (imat.GT.0) THEN
!           Current character in TEXT matches one (or more) of the
!           characters in ch.
            IF (prev) THEN
              IF (T2.LT.Tmax) THEN
!               Keep searching for a block of text
                T2=T2+1
                T1=T2
              ELSE
!               Did not find any text blocks before running
!               out of characters in TEXT.
                again=.false.
                iflg=1
              ENDIF
            ELSE
!             Previous character did not match ch, so terminate.
!             NOTE: This is "NORMAL" termination of the loop
              again=.false.
              T2=T2-1
              iflg = 0
            ENDIF
          ELSEIF (T2.LT.Tmax) THEN
!           Add a letter to the current block of text
            prev = .false.
            T2=T2+1
          ELSE
!           Reached the end of the characters in TEXT before reaching
!           another delimiting character.  A text block was identified
!           however.
            again=.false.
            iflg=-1
          ENDIF
        ENDDO

        IF (iflg.EQ.0) THEN
!         Copy characters into WORD and set col for return
          CALL CopyTxt(1,Text(T1:T2),Word)
          col = T2+1
        ELSE
!         Copy characters into WORD and set col for return
          CALL CopyTxt(1,Text(T1:T2),Word)
          col = 1
        ENDIF
      ENDIF

      END SUBROUTINE TxtExt
!EOC
      END MODULE CHARPAK_MOD
